#include <Arduino_LSM6DS3.h> //https://www.arduino.cc/reference/en/libraries/arduino_lsm6ds3/

//DIY filter with circular buffer:
const int BUFFER_SIZE = 10;
float sensorBuffer[BUFFER_SIZE];
int bufferPosition = 0;

//Kalman filter library:
#include <SimpleKalmanFilter.h> //https://github.com/denyssene/SimpleKalmanFilter
SimpleKalmanFilter simpleKalmanFilter(2, 2, 0.01); //Is there a bug in this? I get weird spikes like a /0 error. Leaving in to check

//Exponentially weighted average:
//from github: output = (alpha * reading + (alphaScale - alpha) * lastOutput) / alphaScale
#include <Ewma.h>
Ewma accelFilter1(0.1);   // Less smoothing - faster to detect changes, but more prone to noise
Ewma accelFilter2(0.01);  // More smoothing - less prone to noise, but slower to detect changes

//These would be packaged into a class:
float peak= 0;
int holdTime = 1000; //ms
long int lastPeakTime = 0;
float decay = .95;


void setup() {
  Serial.begin(576000);
  while(!Serial);
  // start the IMU:
  if (!IMU.begin()) {
    Serial.println("Failed to initialize IMU");
    // stop here if you can't access the IMU:
    while (true);
  }
}
 
void loop() {
  // values for acceleration &amp; rotation:
  float xAcc, yAcc, zAcc;
  float xGyro, yGyro, zGyro;

  float scale = 20.0; //just for the serial plotter - to make it show up larger for class
 
  // if both accelerometer and gyrometer are ready to be read:
  if (IMU.accelerationAvailable() &&
      IMU.gyroscopeAvailable()) {      
    // read accelerometer and gyrometer:
    IMU.readAcceleration(yAcc, xAcc, zAcc); //I did the example with the x-axis, but the cable makes it easier to move on the y, so I just swapped here rather than change all below...
    IMU.readGyroscope(yGyro, xGyro, zGyro);
    xAcc *= scale;

    //update buffer and average:
    addValueToBuffer(xAcc);
    float xAvg = calculateAverageInBuffer();

    //update Kalman (bugs?)
    float estimated_value = simpleKalmanFilter.updateEstimate(xAcc);

    //update weighted filter
    float filtered1 = accelFilter1.filter(xAcc);
    float filtered2 = accelFilter2.filter(xAcc);

    //update Peak detection 
    updatePeakDetector(xAcc);
    
    // print the results:
    Serial.print("xAcc:");         //"Raw" X accel value
    Serial.print(xAcc);         //"Raw" X accel value
    Serial.print(",");
    Serial.print("xAvg:");         //"Raw" X accel value
    Serial.print(xAvg);         //DIY Averaged
    Serial.print(",");
    Serial.print("FastF:");         //"Raw" X accel value
    Serial.print(filtered1);    //Exponentially weighted moving average - fast
    Serial.print(",");
    Serial.print("SlowF:");         //"Raw" X accel value
    Serial.print(filtered2);    //Exponentially weighted moving average - slow
    Serial.print(",");
    Serial.print("peak:");         //"Raw" X accel value
    Serial.print(peak);         //DIY peak detector
    //Serial.print(",");
    
//    Serial.print(estimated_value + 2);
//    Serial.print(yAcc);
//    Serial.print(",");
//    Serial.print(zAcc);
//    Serial.print(",");
//    Serial.print(xGyro);
//    Serial.print(",");
//    Serial.print(yGyro);
//    Serial.print(",");
//    Serial.print(zGyro);
    
    Serial.println();
    delay(20);
  }
}

void addValueToBuffer(float v) {
  sensorBuffer[bufferPosition] = v;
  bufferPosition++;
  if (bufferPosition > BUFFER_SIZE) bufferPosition = 0;
}

float calculateAverageInBuffer() {
  float result = 0;
  for (int i=0; i<BUFFER_SIZE; i++) {
    result += sensorBuffer[i];
  }
  return result / BUFFER_SIZE;
}

void updatePeakDetector(float v) {
  long int currentTime = millis();
  if (v>peak) {
    peak = v;
    lastPeakTime = currentTime;
  } else {
    if (currentTime > lastPeakTime + holdTime) {
      peak *= decay;
    }
  }
}
